# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 1.27.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GeographicCounts(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'outcome_count_by_zip': 'list[str]',
        'outcome_count_by_state': 'list[str]',
        'intervention_count_by_zip': 'list[str]',
        'intervention_count_by_state': 'list[str]',
        'cohort_count_by_zip': 'list[str]',
        'cohort_count_by_state': 'list[str]'
    }

    attribute_map = {
        'outcome_count_by_zip': 'outcomeCountByZip',
        'outcome_count_by_state': 'outcomeCountByState',
        'intervention_count_by_zip': 'interventionCountByZip',
        'intervention_count_by_state': 'interventionCountByState',
        'cohort_count_by_zip': 'cohortCountByZip',
        'cohort_count_by_state': 'cohortCountByState'
    }

    def __init__(self, outcome_count_by_zip=None, outcome_count_by_state=None, intervention_count_by_zip=None, intervention_count_by_state=None, cohort_count_by_zip=None, cohort_count_by_state=None):  # noqa: E501
        """GeographicCounts - a model defined in Swagger"""  # noqa: E501
        self._outcome_count_by_zip = None
        self._outcome_count_by_state = None
        self._intervention_count_by_zip = None
        self._intervention_count_by_state = None
        self._cohort_count_by_zip = None
        self._cohort_count_by_state = None
        self.discriminator = None
        if outcome_count_by_zip is not None:
            self.outcome_count_by_zip = outcome_count_by_zip
        if outcome_count_by_state is not None:
            self.outcome_count_by_state = outcome_count_by_state
        if intervention_count_by_zip is not None:
            self.intervention_count_by_zip = intervention_count_by_zip
        if intervention_count_by_state is not None:
            self.intervention_count_by_state = intervention_count_by_state
        if cohort_count_by_zip is not None:
            self.cohort_count_by_zip = cohort_count_by_zip
        if cohort_count_by_state is not None:
            self.cohort_count_by_state = cohort_count_by_state

    @property
    def outcome_count_by_zip(self):
        """Gets the outcome_count_by_zip of this GeographicCounts.  # noqa: E501


        :return: The outcome_count_by_zip of this GeographicCounts.  # noqa: E501
        :rtype: list[str]
        """
        return self._outcome_count_by_zip

    @outcome_count_by_zip.setter
    def outcome_count_by_zip(self, outcome_count_by_zip):
        """Sets the outcome_count_by_zip of this GeographicCounts.


        :param outcome_count_by_zip: The outcome_count_by_zip of this GeographicCounts.  # noqa: E501
        :type: list[str]
        """

        self._outcome_count_by_zip = outcome_count_by_zip

    @property
    def outcome_count_by_state(self):
        """Gets the outcome_count_by_state of this GeographicCounts.  # noqa: E501


        :return: The outcome_count_by_state of this GeographicCounts.  # noqa: E501
        :rtype: list[str]
        """
        return self._outcome_count_by_state

    @outcome_count_by_state.setter
    def outcome_count_by_state(self, outcome_count_by_state):
        """Sets the outcome_count_by_state of this GeographicCounts.


        :param outcome_count_by_state: The outcome_count_by_state of this GeographicCounts.  # noqa: E501
        :type: list[str]
        """

        self._outcome_count_by_state = outcome_count_by_state

    @property
    def intervention_count_by_zip(self):
        """Gets the intervention_count_by_zip of this GeographicCounts.  # noqa: E501


        :return: The intervention_count_by_zip of this GeographicCounts.  # noqa: E501
        :rtype: list[str]
        """
        return self._intervention_count_by_zip

    @intervention_count_by_zip.setter
    def intervention_count_by_zip(self, intervention_count_by_zip):
        """Sets the intervention_count_by_zip of this GeographicCounts.


        :param intervention_count_by_zip: The intervention_count_by_zip of this GeographicCounts.  # noqa: E501
        :type: list[str]
        """

        self._intervention_count_by_zip = intervention_count_by_zip

    @property
    def intervention_count_by_state(self):
        """Gets the intervention_count_by_state of this GeographicCounts.  # noqa: E501


        :return: The intervention_count_by_state of this GeographicCounts.  # noqa: E501
        :rtype: list[str]
        """
        return self._intervention_count_by_state

    @intervention_count_by_state.setter
    def intervention_count_by_state(self, intervention_count_by_state):
        """Sets the intervention_count_by_state of this GeographicCounts.


        :param intervention_count_by_state: The intervention_count_by_state of this GeographicCounts.  # noqa: E501
        :type: list[str]
        """

        self._intervention_count_by_state = intervention_count_by_state

    @property
    def cohort_count_by_zip(self):
        """Gets the cohort_count_by_zip of this GeographicCounts.  # noqa: E501


        :return: The cohort_count_by_zip of this GeographicCounts.  # noqa: E501
        :rtype: list[str]
        """
        return self._cohort_count_by_zip

    @cohort_count_by_zip.setter
    def cohort_count_by_zip(self, cohort_count_by_zip):
        """Sets the cohort_count_by_zip of this GeographicCounts.


        :param cohort_count_by_zip: The cohort_count_by_zip of this GeographicCounts.  # noqa: E501
        :type: list[str]
        """

        self._cohort_count_by_zip = cohort_count_by_zip

    @property
    def cohort_count_by_state(self):
        """Gets the cohort_count_by_state of this GeographicCounts.  # noqa: E501


        :return: The cohort_count_by_state of this GeographicCounts.  # noqa: E501
        :rtype: list[str]
        """
        return self._cohort_count_by_state

    @cohort_count_by_state.setter
    def cohort_count_by_state(self, cohort_count_by_state):
        """Sets the cohort_count_by_state of this GeographicCounts.


        :param cohort_count_by_state: The cohort_count_by_state of this GeographicCounts.  # noqa: E501
        :type: list[str]
        """

        self._cohort_count_by_state = cohort_count_by_state

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GeographicCounts, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GeographicCounts):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from urllib.parse import urlparse
import pyarrow.dataset as ds
import boto3


def get_parquet_file_metadata(s3_uri):
    dataset = ds.dataset(s3_uri)
    n_columns = len(dataset.schema.names)
    total_n_rows = 0
    for batch in dataset.to_batches(columns=[dataset.schema.names[0]], batch_size=500000, use_async=True):
        total_n_rows += batch.num_rows
    return n_columns, total_n_rows


def get_metadata(s3_uri):
    s3_client = boto3.client("s3")
    bucket, key = extract_bucket_and_key(s3_uri)
    metadata = s3_client.head_object(
        Bucket=bucket,
        Key=key
    )
    return metadata


def s3_listdir(s3_uri, max_list_len=1000):
    """
    Copied from technician's adviser
    Replicates os.listdir() functionality for boto3. Returns a list of all objects underneath the
    folder `directory_name` in Bucket `bucket_name`. To be clear, S3 has no folders. We actually
    return all objects in the top-level bucket with the `directory_name` prefix, but the apparent
    functionality is the same to the end-user.

    :param s3_uri: S3 uri to list
    :param max_list_len: Max items to return in initial list

    :return: list of strings giving the names of all objects in the directory `directory_name`
    """
    client = boto3.client("s3")
    bucket_name, directory_name = extract_bucket_and_key(s3_uri)
    is_truncated = True
    continuation_token = None
    keys = []
    while is_truncated:
        if continuation_token is None:
            response = client.list_objects_v2(Bucket=bucket_name, Prefix=directory_name, MaxKeys=max_list_len)
            if response['KeyCount'] == 0:
                return []
        else:
            response = client.list_objects_v2(Bucket=bucket_name,
                                              Prefix=directory_name,
                                              MaxKeys=max_list_len,
                                              ContinuationToken=continuation_token)
        is_truncated = response.get('IsTruncated', False)
        continuation_token = response.get('NextContinuationToken', None)
        keys += [obj.get('Key') for obj in response['Contents']]

    return [f"s3://{bucket_name}/{key}" for key in keys]


def extract_bucket_and_key(s3_uri):
    parsed_url = urlparse(s3_uri)
    bucket_name = parsed_url.netloc
    directory_name = parsed_url.path[1:]
    return bucket_name, directory_name




# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 1.27.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Project(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'name': 'str',
        'description': 'str',
        'organization_id': 'str',
        'parent_project_id': 'str',
        'project_level': 'float',
        'children_projects': 'list[Project]',
        'models': 'list[Model]',
        'model_batches': 'list[ModelBatch]',
        'model_batch_jobs': 'list[ModelBatchJob]',
        'members': 'list[ProjectMember]',
        'features': 'list[Feature]',
        'model_jobs': 'list[ModelJob]',
        'model_job_events': 'list[ModelJobEvent]',
        'model_job_statuses': 'list[ModelJobStatus]',
        'analyses': 'list[Analysis]',
        'analysis_jobs': 'list[AnalysisJob]',
        'analysis_job_statuses': 'list[AnalysisJobStatus]',
        'processes': 'list[Process]',
        'process_jobs': 'list[ProcessJob]',
        'process_job_statuses': 'list[ProcessJobStatus]',
        'user_favorites': 'list[UserFavorite]',
        'last_updated_by': 'str',
        'created_by': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'description': 'description',
        'organization_id': 'organizationId',
        'parent_project_id': 'parentProjectId',
        'project_level': 'projectLevel',
        'children_projects': 'childrenProjects',
        'models': 'models',
        'model_batches': 'modelBatches',
        'model_batch_jobs': 'modelBatchJobs',
        'members': 'members',
        'features': 'features',
        'model_jobs': 'modelJobs',
        'model_job_events': 'modelJobEvents',
        'model_job_statuses': 'modelJobStatuses',
        'analyses': 'analyses',
        'analysis_jobs': 'analysisJobs',
        'analysis_job_statuses': 'analysisJobStatuses',
        'processes': 'processes',
        'process_jobs': 'processJobs',
        'process_job_statuses': 'processJobStatuses',
        'user_favorites': 'userFavorites',
        'last_updated_by': 'lastUpdatedBy',
        'created_by': 'createdBy',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'version': 'version'
    }

    def __init__(self, id=None, name=None, description=None, organization_id=None, parent_project_id=None, project_level=None, children_projects=None, models=None, model_batches=None, model_batch_jobs=None, members=None, features=None, model_jobs=None, model_job_events=None, model_job_statuses=None, analyses=None, analysis_jobs=None, analysis_job_statuses=None, processes=None, process_jobs=None, process_job_statuses=None, user_favorites=None, last_updated_by=None, created_by=None, created_at=None, updated_at=None, version=None):  # noqa: E501
        """Project - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._name = None
        self._description = None
        self._organization_id = None
        self._parent_project_id = None
        self._project_level = None
        self._children_projects = None
        self._models = None
        self._model_batches = None
        self._model_batch_jobs = None
        self._members = None
        self._features = None
        self._model_jobs = None
        self._model_job_events = None
        self._model_job_statuses = None
        self._analyses = None
        self._analysis_jobs = None
        self._analysis_job_statuses = None
        self._processes = None
        self._process_jobs = None
        self._process_job_statuses = None
        self._user_favorites = None
        self._last_updated_by = None
        self._created_by = None
        self._created_at = None
        self._updated_at = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        self.name = name
        if description is not None:
            self.description = description
        if organization_id is not None:
            self.organization_id = organization_id
        if parent_project_id is not None:
            self.parent_project_id = parent_project_id
        if project_level is not None:
            self.project_level = project_level
        if children_projects is not None:
            self.children_projects = children_projects
        if models is not None:
            self.models = models
        if model_batches is not None:
            self.model_batches = model_batches
        if model_batch_jobs is not None:
            self.model_batch_jobs = model_batch_jobs
        if members is not None:
            self.members = members
        if features is not None:
            self.features = features
        if model_jobs is not None:
            self.model_jobs = model_jobs
        if model_job_events is not None:
            self.model_job_events = model_job_events
        if model_job_statuses is not None:
            self.model_job_statuses = model_job_statuses
        if analyses is not None:
            self.analyses = analyses
        if analysis_jobs is not None:
            self.analysis_jobs = analysis_jobs
        if analysis_job_statuses is not None:
            self.analysis_job_statuses = analysis_job_statuses
        if processes is not None:
            self.processes = processes
        if process_jobs is not None:
            self.process_jobs = process_jobs
        if process_job_statuses is not None:
            self.process_job_statuses = process_job_statuses
        if user_favorites is not None:
            self.user_favorites = user_favorites
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if created_by is not None:
            self.created_by = created_by
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this Project.  # noqa: E501


        :return: The id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Project.


        :param id: The id of this Project.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this Project.  # noqa: E501


        :return: The name of this Project.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Project.


        :param name: The name of this Project.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def description(self):
        """Gets the description of this Project.  # noqa: E501


        :return: The description of this Project.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Project.


        :param description: The description of this Project.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def organization_id(self):
        """Gets the organization_id of this Project.  # noqa: E501


        :return: The organization_id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this Project.


        :param organization_id: The organization_id of this Project.  # noqa: E501
        :type: str
        """

        self._organization_id = organization_id

    @property
    def parent_project_id(self):
        """Gets the parent_project_id of this Project.  # noqa: E501


        :return: The parent_project_id of this Project.  # noqa: E501
        :rtype: str
        """
        return self._parent_project_id

    @parent_project_id.setter
    def parent_project_id(self, parent_project_id):
        """Sets the parent_project_id of this Project.


        :param parent_project_id: The parent_project_id of this Project.  # noqa: E501
        :type: str
        """

        self._parent_project_id = parent_project_id

    @property
    def project_level(self):
        """Gets the project_level of this Project.  # noqa: E501


        :return: The project_level of this Project.  # noqa: E501
        :rtype: float
        """
        return self._project_level

    @project_level.setter
    def project_level(self, project_level):
        """Sets the project_level of this Project.


        :param project_level: The project_level of this Project.  # noqa: E501
        :type: float
        """

        self._project_level = project_level

    @property
    def children_projects(self):
        """Gets the children_projects of this Project.  # noqa: E501


        :return: The children_projects of this Project.  # noqa: E501
        :rtype: list[Project]
        """
        return self._children_projects

    @children_projects.setter
    def children_projects(self, children_projects):
        """Sets the children_projects of this Project.


        :param children_projects: The children_projects of this Project.  # noqa: E501
        :type: list[Project]
        """

        self._children_projects = children_projects

    @property
    def models(self):
        """Gets the models of this Project.  # noqa: E501


        :return: The models of this Project.  # noqa: E501
        :rtype: list[Model]
        """
        return self._models

    @models.setter
    def models(self, models):
        """Sets the models of this Project.


        :param models: The models of this Project.  # noqa: E501
        :type: list[Model]
        """

        self._models = models

    @property
    def model_batches(self):
        """Gets the model_batches of this Project.  # noqa: E501


        :return: The model_batches of this Project.  # noqa: E501
        :rtype: list[ModelBatch]
        """
        return self._model_batches

    @model_batches.setter
    def model_batches(self, model_batches):
        """Sets the model_batches of this Project.


        :param model_batches: The model_batches of this Project.  # noqa: E501
        :type: list[ModelBatch]
        """

        self._model_batches = model_batches

    @property
    def model_batch_jobs(self):
        """Gets the model_batch_jobs of this Project.  # noqa: E501


        :return: The model_batch_jobs of this Project.  # noqa: E501
        :rtype: list[ModelBatchJob]
        """
        return self._model_batch_jobs

    @model_batch_jobs.setter
    def model_batch_jobs(self, model_batch_jobs):
        """Sets the model_batch_jobs of this Project.


        :param model_batch_jobs: The model_batch_jobs of this Project.  # noqa: E501
        :type: list[ModelBatchJob]
        """

        self._model_batch_jobs = model_batch_jobs

    @property
    def members(self):
        """Gets the members of this Project.  # noqa: E501


        :return: The members of this Project.  # noqa: E501
        :rtype: list[ProjectMember]
        """
        return self._members

    @members.setter
    def members(self, members):
        """Sets the members of this Project.


        :param members: The members of this Project.  # noqa: E501
        :type: list[ProjectMember]
        """

        self._members = members

    @property
    def features(self):
        """Gets the features of this Project.  # noqa: E501


        :return: The features of this Project.  # noqa: E501
        :rtype: list[Feature]
        """
        return self._features

    @features.setter
    def features(self, features):
        """Sets the features of this Project.


        :param features: The features of this Project.  # noqa: E501
        :type: list[Feature]
        """

        self._features = features

    @property
    def model_jobs(self):
        """Gets the model_jobs of this Project.  # noqa: E501


        :return: The model_jobs of this Project.  # noqa: E501
        :rtype: list[ModelJob]
        """
        return self._model_jobs

    @model_jobs.setter
    def model_jobs(self, model_jobs):
        """Sets the model_jobs of this Project.


        :param model_jobs: The model_jobs of this Project.  # noqa: E501
        :type: list[ModelJob]
        """

        self._model_jobs = model_jobs

    @property
    def model_job_events(self):
        """Gets the model_job_events of this Project.  # noqa: E501


        :return: The model_job_events of this Project.  # noqa: E501
        :rtype: list[ModelJobEvent]
        """
        return self._model_job_events

    @model_job_events.setter
    def model_job_events(self, model_job_events):
        """Sets the model_job_events of this Project.


        :param model_job_events: The model_job_events of this Project.  # noqa: E501
        :type: list[ModelJobEvent]
        """

        self._model_job_events = model_job_events

    @property
    def model_job_statuses(self):
        """Gets the model_job_statuses of this Project.  # noqa: E501


        :return: The model_job_statuses of this Project.  # noqa: E501
        :rtype: list[ModelJobStatus]
        """
        return self._model_job_statuses

    @model_job_statuses.setter
    def model_job_statuses(self, model_job_statuses):
        """Sets the model_job_statuses of this Project.


        :param model_job_statuses: The model_job_statuses of this Project.  # noqa: E501
        :type: list[ModelJobStatus]
        """

        self._model_job_statuses = model_job_statuses

    @property
    def analyses(self):
        """Gets the analyses of this Project.  # noqa: E501


        :return: The analyses of this Project.  # noqa: E501
        :rtype: list[Analysis]
        """
        return self._analyses

    @analyses.setter
    def analyses(self, analyses):
        """Sets the analyses of this Project.


        :param analyses: The analyses of this Project.  # noqa: E501
        :type: list[Analysis]
        """

        self._analyses = analyses

    @property
    def analysis_jobs(self):
        """Gets the analysis_jobs of this Project.  # noqa: E501


        :return: The analysis_jobs of this Project.  # noqa: E501
        :rtype: list[AnalysisJob]
        """
        return self._analysis_jobs

    @analysis_jobs.setter
    def analysis_jobs(self, analysis_jobs):
        """Sets the analysis_jobs of this Project.


        :param analysis_jobs: The analysis_jobs of this Project.  # noqa: E501
        :type: list[AnalysisJob]
        """

        self._analysis_jobs = analysis_jobs

    @property
    def analysis_job_statuses(self):
        """Gets the analysis_job_statuses of this Project.  # noqa: E501


        :return: The analysis_job_statuses of this Project.  # noqa: E501
        :rtype: list[AnalysisJobStatus]
        """
        return self._analysis_job_statuses

    @analysis_job_statuses.setter
    def analysis_job_statuses(self, analysis_job_statuses):
        """Sets the analysis_job_statuses of this Project.


        :param analysis_job_statuses: The analysis_job_statuses of this Project.  # noqa: E501
        :type: list[AnalysisJobStatus]
        """

        self._analysis_job_statuses = analysis_job_statuses

    @property
    def processes(self):
        """Gets the processes of this Project.  # noqa: E501


        :return: The processes of this Project.  # noqa: E501
        :rtype: list[Process]
        """
        return self._processes

    @processes.setter
    def processes(self, processes):
        """Sets the processes of this Project.


        :param processes: The processes of this Project.  # noqa: E501
        :type: list[Process]
        """

        self._processes = processes

    @property
    def process_jobs(self):
        """Gets the process_jobs of this Project.  # noqa: E501


        :return: The process_jobs of this Project.  # noqa: E501
        :rtype: list[ProcessJob]
        """
        return self._process_jobs

    @process_jobs.setter
    def process_jobs(self, process_jobs):
        """Sets the process_jobs of this Project.


        :param process_jobs: The process_jobs of this Project.  # noqa: E501
        :type: list[ProcessJob]
        """

        self._process_jobs = process_jobs

    @property
    def process_job_statuses(self):
        """Gets the process_job_statuses of this Project.  # noqa: E501


        :return: The process_job_statuses of this Project.  # noqa: E501
        :rtype: list[ProcessJobStatus]
        """
        return self._process_job_statuses

    @process_job_statuses.setter
    def process_job_statuses(self, process_job_statuses):
        """Sets the process_job_statuses of this Project.


        :param process_job_statuses: The process_job_statuses of this Project.  # noqa: E501
        :type: list[ProcessJobStatus]
        """

        self._process_job_statuses = process_job_statuses

    @property
    def user_favorites(self):
        """Gets the user_favorites of this Project.  # noqa: E501


        :return: The user_favorites of this Project.  # noqa: E501
        :rtype: list[UserFavorite]
        """
        return self._user_favorites

    @user_favorites.setter
    def user_favorites(self, user_favorites):
        """Sets the user_favorites of this Project.


        :param user_favorites: The user_favorites of this Project.  # noqa: E501
        :type: list[UserFavorite]
        """

        self._user_favorites = user_favorites

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this Project.  # noqa: E501


        :return: The last_updated_by of this Project.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this Project.


        :param last_updated_by: The last_updated_by of this Project.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def created_by(self):
        """Gets the created_by of this Project.  # noqa: E501


        :return: The created_by of this Project.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this Project.


        :param created_by: The created_by of this Project.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def created_at(self):
        """Gets the created_at of this Project.  # noqa: E501


        :return: The created_at of this Project.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Project.


        :param created_at: The created_at of this Project.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this Project.  # noqa: E501


        :return: The updated_at of this Project.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Project.


        :param updated_at: The updated_at of this Project.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def version(self):
        """Gets the version of this Project.  # noqa: E501


        :return: The version of this Project.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this Project.


        :param version: The version of this Project.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Project, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Project):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
